#!/usr/bin/env python

# Developed by Sebastien DAMAYE (aldeid.com) in the shape of forensicscontest.com
# (Puzzle #4: The Curious Mr. X). Feel free to modify.
# History:
# 2010-03-27 08:14, Window size added. Version: 1.1
# 2010-02-14 21:42, Initial rev. Version: 1.0
#
# This script uses a MySQL connection. Please first make sure that you
# have created necessary objects and privileges, using following SQL
# script provided with pyScanXtract package (pyscan.sql)
# (Notice that DROP priv is required to TRUNCATE table unless
# --dont-purge option is used)
#
# Check that MySQL is started before you launch the script.
#
# In addition, this script needs a OUI file to detect mac vendors.
# The complete updated database (txt format) can be downloaded from
# http://standards.ieee.org/regauth/oui/oui.txt
#

from optparse import OptionParser
import sys
import shutil
import pcapy
import impacket.ImpactDecoder as Decoders
import impacket.ImpactPacket as Packets
import os.path
import os
import MySQLdb
import datetime
import struct
import hashlib

class PyScan:

    ###
    # You can modify these parameters to fit with your configuration
    #
    DBHOST = '127.0.0.1'    # Host to connect to
    DBUSER = 'pyscan'       # User name to connect to database
    DBPSWD = 'pyscan'       # Password to connect to database
    DBNAME = 'pyscan'       # Database name
    BASEWB = 'http://localhost/pyscan' # Base path for pyscan web interface
                                       # Don't put / at the end of the path
                                       # e.g. http://localhost/pyscan

    def __init__(self, pcapfile, reportpath="./report", ouifile="./oui.txt", dont_purge=None):
        """Performs some checks and connects to database
        """
        # Checks if pcap file exists
        assert pcapfile
        self.pcapfile = pcapfile
        if not os.path.exists(pcapfile):
            raise TypeError("Pcap file not found. Please check location.")

        self.reportpath = reportpath
        if not os.path.exists(self.reportpath):
            os.makedirs(self.reportpath)

        pc = open(pcapfile, mode='rb')
        pcheader = pc.read(24)

        # Read the first 4 bytes for the magic number, determine endianness
        pcmagicnum = struct.unpack("I", pcheader[0:4])[0]
        if pcmagicnum != 0xd4c3b2a1:
            # Little endian
            pcendflag = "<"
        elif magicnum == 0xa1b2c3d4:
            # Big endign
            pcendflag = ">"
        else:
            raise Exception('Specified file is not a libpcap capture')
        
        pcaph = struct.unpack("%sIHHIIII"%pcendflag, pcheader)
        if pcaph[1] != 2 and pcaph[2] != 4 \
                and pcaph[3] != 0 and pcaph[4] != 0 \
                and pcaph[5] != 65535:
            raise Exception('Unsupported pcap header format or version')

        # Checks if OUI file exists
        assert ouifile
        if not os.path.exists(ouifile):
            raise TypeError("OUI file not found. Please check location."
            + "OUI file can be downloaded from http://standards.ieee.org/regauth/oui/oui.txt")
        self.ouifile = ouifile

        # Connection to MySQL database
        try:
            self.conn = MySQLdb.connect (
                host = self.DBHOST,
                user = self.DBUSER,
                passwd = self.DBPSWD,
                db = self.DBNAME)
            self.cursor = self.conn.cursor ()
        except MySQLdb.Error, e:
            print "Error %d: %s" % (e.args[0], e.args[1])
            sys.exit (1)
        
        # Option: purge table
        if not dont_purge:
            self.cursor.execute ("TRUNCATE TABLE pyscan")

        # Initial list of hosts
        self.hosts = {}

    def md5sum(self, myfile):
        m = hashlib.md5()
        f = file(myfile, 'rb')
        while True:
            t = f.read(1024)
            if len(t) == 0:
                break
            m.update(t)
        return m.hexdigest()

    def writefile(self, f, content):
        """Dump content in a file
        """
        obFile = open(os.path.join(self.reportpath, f), 'a')
        obFile.write(content)
        obFile.close()

    def decodemac(self, mac):
        """Decode mac address with hex notation
        """
        m = ''
        for i in mac:
            t = "%x" % i
            if len(t)==1:
                t = '0'+t
            m=m+":"+t
        return m[1:]

    def countnumberframes(self):
        """Returns number of frames contained in pcap file
        """
        f = pcapy.open_offline(self.pcapfile)
        c = 0
        while f.next()[1]!="":
            c+=1
        return c

    def macvendor(self, mac):
        """Finds mac vendor in OUI file from 6 first cars of mac addr
        """
        #Only 6 first cars are necessary in upper cap
        pref = mac.replace(':', '')[:6].upper()
        f = open(self.ouifile, 'r')
        while 1:
                li = f.readline()
                if li=='':
                        break
                if pref in li:
                        return (li.replace('\n', '').replace('\t', ''))[20:]
                        break
        f.close()

    def getframefromflow(self, ip_src, sport, ip_dst, dport, proto):
        """Checks whether flow already exists in DB
           Returns frame number or zero
        """
        self.cursor.execute ("""
            SELECT snd_frame, ip_src FROM pyscan
            WHERE (ip_src=%s AND ip_dst=%s AND sport=%s AND dport=%s)
            OR (ip_src=%s AND ip_dst=%s AND sport=%s AND dport=%s)
            AND proto=%s
            """, (ip_src, ip_dst, sport, dport,
                ip_dst, ip_src, dport, sport, proto))
        row = self.cursor.fetchone()
        if row == None:
            row = [0, 0]
        return row

    def main(self):
        """Stores and reassemble frames in MySQL database.
           IP / TCP / UDP / ICMP protocols are handled in this version
        """
        print "Analysing pcap file..."
        reader = pcapy.open_offline(self.pcapfile)

        # Decoders for ethernet, IP, TCP, UDP and ICMP
        eth_decoder = Decoders.EthDecoder()
        ip_decoder = Decoders.IPDecoder()
        tcp_decoder = Decoders.TCPDecoder()
        udp_decoder = Decoders.UDPDecoder()
        icmp_decoder = Decoders.ICMPDecoder()
        ip_icmp_decoder = Decoders.IPDecoderForICMP()

        # Initial frame counter
        frame = 0
        nFrames = self.countnumberframes()

        (header, payload) = reader.next()

        while frame < nFrames:

            frame+=1
            # Timestamp
            (ts, usec) = header.getts()

            if frame==1:
                # First frame = reference (0 second)
                reftime = float(str(ts)+"."+str(usec))
                tm = 0
                # First Frame TimeStamp
                self.ffts = datetime.datetime.fromtimestamp(ts).strftime("%Y-%m-%d %H:%M:%S")
            else:
                # Number of seconds since beginning of capture
                tm = float(str(ts)+"."+str(usec)) - reftime
                # Last frame time stamp
                if frame == nFrames:
                    self.lfts = datetime.datetime.fromtimestamp(ts).strftime("%Y-%m-%d %H:%M:%S")

            # Progress bar
            p = int(float(frame)/nFrames*50)
            sys.stdout.write("\r["+"#"*p+" "*(50-p)+"] "+str(p*2)+"% ("
                +str(frame)+"/"+str(nFrames)+")")
            sys.stdout.flush()

            #---------------------------------
            #           ETHERNET LAYER
            #---------------------------------
            ethernet = eth_decoder.decode(payload)
            
            #---------------------------------
            #           IP PACKETS
            #---------------------------------
            if ethernet.get_ether_type() == Packets.IP.ethertype:

                ip = ip_decoder.decode(payload[ethernet.get_header_size():])

                # IP header information
                ip_ihl    = ip.get_ip_hl()
                ip_tos    = ip.get_ip_tos()
                ip_len    = ip.get_ip_len()
                ip_id     = ip.get_ip_id()
                ip_ttl    = ip.get_ip_ttl()
                if ip.get_ip_df() > 0:
                    df = '1'
                else:
                    df = '0'
                if ip.get_ip_mf() > 0:
                    mf = '1'
                else:
                    mf = '0'
                ip_flags  = str(ip.get_ip_rf()) + df + mf
                ip_src    = ip.get_ip_src()
                ip_dst    = ip.get_ip_dst()

                #---------------------------------
                #           TCP PACKETS
                #---------------------------------
                if ip.get_ip_p() == Packets.TCP.protocol:
                    tcp = tcp_decoder.decode(
                        payload[ethernet.get_header_size()+ip.get_header_size():])

                    # New hosts are added to the list of hosts
                    if self.hosts.keys().count(ip_src)==0:
                        self.hosts[ip_src] = self.decodemac(ethernet.get_ether_shost())
                    if self.hosts.keys().count(ip_dst)==0:
                        self.hosts[ip_dst] = self.decodemac(ethernet.get_ether_dhost())

                    # TCP header information
                    tcp_sport = tcp.get_th_sport()
                    tcp_dport = tcp.get_th_dport()
                    tcp_ece   = tcp.get_ECE()
                    tcp_cwr   = tcp.get_CWR()
                    tcp_urg   = tcp.get_URG()
                    tcp_ack   = tcp.get_ACK()
                    tcp_psh   = tcp.get_PSH()
                    tcp_rst   = tcp.get_RST()
                    tcp_syn   = tcp.get_SYN()
                    tcp_fin   = tcp.get_FIN()
                    seq       = tcp.get_th_seq()
                    ack       = tcp.get_th_ack()
                    win       = tcp.get_th_win()

                    # Checks if src:sport->dst:dport or dst:dport->src:sport
                    # doesn't already exist in DB
                    arrFrame = self.getframefromflow(ip_src, tcp_sport, ip_dst, tcp_dport, 'TCP')
                    if arrFrame[0]==0:
                        # Frame hasn't been found: new flow
                        self.cursor.execute ("""
                            INSERT INTO pyscan(ts, snd_frame, ip_ihl, ip_tos,
                              ip_len, ip_id, ip_flags, ip_ttl, proto, ip_src,
                              ip_dst, sport, dport, snd_tcp_ece, snd_tcp_cwr,
                              snd_tcp_urg, snd_tcp_ack, snd_tcp_psh, snd_tcp_rst,
                              snd_tcp_syn, snd_tcp_fin, seq, ack, win
                            ) VALUES (
                              %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s,
                              %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s
                            )
                            """, (tm, frame, ip_ihl, ip_tos, ip_len, ip_id,
                            ip_flags, ip_ttl, 'TCP', ip_src, ip_dst, tcp_sport,
                            tcp_dport, tcp_ece, tcp_cwr, tcp_urg, tcp_ack,
                            tcp_psh, tcp_rst, tcp_syn, tcp_fin, seq, ack, win))
                    else:
                        if arrFrame[1]==ip_src:
                            #Attacker->Target: update flow (request)
                            if tcp_rst==1:
                                # RST conn
                                self.cursor.execute ("""
                                    UPDATE pyscan SET rst_conn=1
                                    WHERE snd_frame = %s
                                    """, (arrFrame[0]))
                            else:
                                #ACK conn
                                self.cursor.execute ("""
                                    UPDATE pyscan SET ack_conn=1
                                    WHERE snd_frame = %s
                                    """, (arrFrame[0]))
                        else:
                            # Target->Attacker: update flow (reply)
                            # "snd_frame IS NULL" clause is added to
                            # prevent from updating an already existing answer
                            self.cursor.execute ("""
                                UPDATE pyscan SET
                                    rcv_frame = %s,
                                    rcv_tcp_urg = %s,
                                    rcv_tcp_ack = %s,
                                    rcv_tcp_psh = %s,
                                    rcv_tcp_rst = %s,
                                    rcv_tcp_syn = %s,
                                    rcv_tcp_fin = %s,
                                    rcv_ip_id=%s,
                                    ack = %s
                                WHERE snd_frame = %s
                                AND   rcv_frame IS NULL
                                """, (frame,
                                    tcp_urg, tcp_ack, tcp_psh, tcp_rst, tcp_syn,
                                    tcp_fin, ip_id, ack, arrFrame[0]))

                #---------------------------------
                #           UDP PACKETS
                #---------------------------------
                if ip.get_ip_p() == Packets.UDP.protocol:
                    udp = udp_decoder.decode(
                        payload[ethernet.get_header_size()+ip.get_header_size():])

                    # UDP header information
                    udp_sport = udp.get_uh_sport()
                    udp_dport = udp.get_uh_dport()
                    udp_len   = udp.get_size()

                    # Any other way to ignore NBNS (NetBIOS Name Service)
                    # packets with Impacket?
                    if udp_len > 70:
                        # Log SENT UDP PACKETS TO TARGET in DB
                        arrFrame = self.getframefromflow(ip_src, udp_sport, ip_dst, udp_dport, 'UDP')
                        if arrFrame[0]==0:
                            # Frame hasn't been found: new flow
                            self.cursor.execute ("""
                                INSERT INTO pyscan(ts, snd_frame, proto, ip_src,
                                  ip_dst, sport, dport
                                ) VALUES (
                                  %s, %s, %s, %s, %s, %s, %s
                                )
                                """, (tm, frame, 'UDP', ip_src, ip_dst,
                                udp_sport, udp_dport))

                #---------------------------------
                #           ICMP PACKETS
                #---------------------------------
                if ip.get_ip_p() == Packets.ICMP.protocol:
                    icmp = icmp_decoder.decode(
                        payload[ethernet.get_header_size()+ip.get_header_size():])

                    # ICMP header information
                    icmp_type = icmp.get_icmp_type()
                    icmp_code = icmp.get_icmp_code()

                    # ICMP echo request/reply are not kept in database
                    if icmp_type != 8 and icmp_type !=0:

                        # Special IP decoder for ICMP
                        ip_icmp = ip_icmp_decoder.decode(
                            payload[ethernet.get_header_size()+ip.get_header_size()
                            +icmp.get_header_size():])
                        # UDP decoder for IP in ICMP
                        udp_ip_icmp = udp_decoder.decode(
                            payload[ethernet.get_header_size()+ip.get_header_size()
                            +icmp.get_header_size()+ip_icmp.get_header_size():])

                        # IP header information in ICMP
                        ip_icmp_src = ip_icmp.get_ip_src()
                        ip_icmp_dst = ip_icmp.get_ip_dst()

                        # UDP header information in IP in ICMP
                        udp_ip_icmp_sport = udp_ip_icmp.get_uh_sport()
                        udp_ip_icmp_dport = udp_ip_icmp.get_uh_dport()

                        # Looks for frame number that has required criteria
                        fn = self.getframefromflow(ip_icmp_src, udp_ip_icmp_sport,
                            ip_icmp_dst, udp_ip_icmp_dport, 'UDP')[0]

                        # Updates flow in database
                        self.cursor.execute ("""
                            UPDATE pyscan
                            SET rcv_frame=%s, icmp_err=%s
                            WHERE snd_frame=%s
                            """, (frame, icmp_code, fn))

                (header, payload) = reader.next()


    def updatescantypes(self):
        """Update database by guessing scan types
           for each detected scan
        """
        print "\nDetermining scan types..."

        #---------------------------------
        #           SCAN TYPES
        #---------------------------------

        # TCP SYN (only TCP SYN flag activated and no ack.)
        self.cursor.execute ("""
            UPDATE pyscan
            SET    scan_type='TCP SYN'
            WHERE  proto='TCP'
            AND    CONCAT(snd_tcp_ece, snd_tcp_cwr, snd_tcp_urg, snd_tcp_ack,
                   snd_tcp_psh, snd_tcp_rst, snd_tcp_syn, snd_tcp_fin)='00000010'
            AND    ack_conn IS NULL
            """)
        # TCP CONNECT (same as TCP SYN but ack.)
        self.cursor.execute ("""
            UPDATE pyscan
            SET    scan_type='TCP CONNECT'
            WHERE  proto='TCP'
            AND    CONCAT(snd_tcp_ece, snd_tcp_cwr, snd_tcp_urg, snd_tcp_ack,
                   snd_tcp_psh, snd_tcp_rst, snd_tcp_syn, snd_tcp_fin)='00000010'
            AND    ack_conn IS NOT NULL
            """)
        # TCP NULL (no TCP flag activated)
        self.cursor.execute ("""
            UPDATE pyscan
            SET    scan_type='TCP NULL'
            WHERE  proto='TCP'
            AND    CONCAT(snd_tcp_ece, snd_tcp_cwr, snd_tcp_urg, snd_tcp_ack,
                   snd_tcp_psh, snd_tcp_rst, snd_tcp_syn, snd_tcp_fin)='00000000'
            """)
        # TCP FIN (only TCP FIN flag activated)
        self.cursor.execute ("""
            UPDATE pyscan
            SET    scan_type='TCP FIN'
            WHERE  proto='TCP'
            AND    CONCAT(snd_tcp_ece, snd_tcp_cwr, snd_tcp_urg, snd_tcp_ack,
                   snd_tcp_psh, snd_tcp_rst, snd_tcp_syn, snd_tcp_fin)='00000001'
            """)
        # TCP XMAS (only TCP FIN/PSH/URG flags activated)
        self.cursor.execute ("""
            UPDATE pyscan
            SET    scan_type='TCP XMAS'
            WHERE  proto='TCP'
            AND    CONCAT(snd_tcp_ece, snd_tcp_cwr, snd_tcp_urg, snd_tcp_ack,
                   snd_tcp_psh, snd_tcp_rst, snd_tcp_syn, snd_tcp_fin)='00101001'
            """)
        # TCP ACK (only TCP ACK flag activated)
        self.cursor.execute ("""
            UPDATE pyscan
            SET    scan_type='TCP ACK'
            WHERE  proto='TCP'
            AND    CONCAT(snd_tcp_ece, snd_tcp_cwr, snd_tcp_urg, snd_tcp_ack,
                   snd_tcp_psh, snd_tcp_rst, snd_tcp_syn, snd_tcp_fin)='00010000'
            """)
        # TCP RST (only TCP RST flag activated)
        self.cursor.execute ("""
            UPDATE pyscan
            SET    scan_type='TCP RST'
            WHERE  proto='TCP'
            AND    CONCAT(snd_tcp_ece, snd_tcp_cwr, snd_tcp_urg, snd_tcp_ack,
                   snd_tcp_psh, snd_tcp_rst, snd_tcp_syn, snd_tcp_fin)='00000100'
            """)
        # ALL OTHER TCP SCANS (custom flags)
        self.cursor.execute ("""
            UPDATE pyscan
            SET    scan_type='TCP CUSTOM SCAN'
            WHERE  proto='TCP'
            AND    scan_type IS NULL
            """)
        # UDP SCANS
        self.cursor.execute ("""
            UPDATE pyscan
            SET    scan_type='UDP'
            WHERE  proto='UDP'
            """)

        #---------------------------------
        #           PORTS STATE
        #---------------------------------
        print "Determining ports state..."

        #----- TCP SYN -----

        # TCP SYN - OPEN (SYN/ACK from target)
        self.cursor.execute ("""
            UPDATE pyscan
            SET    state='OPEN'
            WHERE  scan_type='TCP SYN'
            AND    rcv_tcp_ack = 1
            AND    rcv_tcp_syn = 1
            """)
        # TCP SYN - CLOSED (RST from target)
        self.cursor.execute ("""
            UPDATE pyscan
            SET    state='CLOSED'
            WHERE  scan_type='TCP SYN'
            AND    rcv_tcp_rst = 1
            """)
        # TCP SYN - FILTERED (no answer from target)
        self.cursor.execute ("""
            UPDATE pyscan
            SET    state='FILTERED'
            WHERE  scan_type='TCP SYN'
            AND    rcv_frame IS NULL
            """)

        #----- TCP CONNECT -----

        # TCP CONNECT - OPEN (SYN/ACK from source)
        self.cursor.execute ("""
            UPDATE pyscan
            SET    state='OPEN'
            WHERE  scan_type='TCP CONNECT'
            AND    rcv_tcp_syn = 1
            AND    rcv_tcp_ack = 1
            """)

        #----- TCP NULL/FIN/XMAS -----

        # TCP NULL/FIN/XMAS - OPEN|FILTERED (no response from target)
        self.cursor.execute ("""
            UPDATE pyscan
            SET    state='OPEN|FILTERED'
            WHERE  scan_type IN ('TCP NULL', 'TCP FIN', 'TCP XMAS')
            AND    rcv_frame IS NULL
            """)
        # TCP NULL/FIN/XMAS - CLOSED (RST received from target)
        self.cursor.execute ("""
            UPDATE pyscan
            SET    state='CLOSED'
            WHERE  scan_type IN ('TCP NULL', 'TCP FIN', 'TCP XMAS')
            AND    rcv_tcp_rst = 1
            """)
        # TCP NULL/FIN/XMAS - FILTERED (ICMP unreachable error)
        self.cursor.execute ("""
            UPDATE pyscan
            SET    state='FILTERED'
            WHERE  scan_type IN ('TCP NULL', 'TCP FIN', 'TCP XMAS')
            AND    icmp_err IN (1, 2, 3, 9, 10, 13)
            """)

        #----- TCP ACK -----

        # TCP ACK - UNFILTERED (RST received from target)
        self.cursor.execute ("""
            UPDATE pyscan
            SET    state='UNFILTERED'
            WHERE  scan_type='TCP ACK'
            AND    rcv_tcp_rst = 1
            """)
        # TCP ACK - FILTERED (no response from target OR ICMP unreachable err)
        self.cursor.execute ("""
            UPDATE pyscan
            SET    state='FILTERED'
            WHERE  (scan_type='TCP ACK'
            AND    rcv_frame IS NULL)
            OR     icmp_err IN (1, 2, 3, 9, 10, 13)
            """)

        #----- UDP SCAN -----
        # UDP SCANS - OPEN|FILTERED (no response from target)
        self.cursor.execute ("""
            UPDATE pyscan
            SET    state='OPEN|FILTERED'
            WHERE  scan_type='UDP'
            AND    rcv_frame IS NULL
            """)
        # UDP SCANS - CLOSED (ICMP port unreachable)
        self.cursor.execute ("""
            UPDATE pyscan
            SET    state='CLOSED'
            WHERE  scan_type='UDP'
            AND    icmp_err=3
            """)
        # UDP SCANS - FILTERED (type 3, code 1, 2, 9, 10 or 13)
        self.cursor.execute ("""
            UPDATE pyscan
            SET    state='FILTERED'
            WHERE  scan_type='UDP'
            AND    icmp_err IN (1, 2, 9, 10, 13)
            """)

        #----- ALL OTHERS: UNDEFINED -----
        self.cursor.execute ("""
            UPDATE pyscan
            SET    state='UNDEFINED'
            WHERE  state IS NULL
            """)


    def exportresults(self):
        print "Generating report..."
        self.writefile("report.html", """<html><head><style>
            h1           { font-size:11pt; padding:2px; }
            h2           { font-size:9pt; padding:2px; }
            body, td, th { font-size:8pt; }
            table        { border-collapse:collapse; border:solid 2px #cccccc; }
            th, td       { border:solid 1px #dddddd;padding:2px;}
            th           { background:#eeeeee; }
            div          { padding:10px; }
            </style></head><body>\n""")
        #===== Index
        self.writefile("report.html", '<div style="border: solid 2px #ccc; \
            background: #eee; text-align:center; font-size: 12pt;"> \
            <a href="#general_information">General information</a> \
            &nbsp;&nbsp;|&nbsp;&nbsp;<a href="#discovered_hosts">Discovered hosts</a> \
            &nbsp;&nbsp;|&nbsp;&nbsp;<a href="#scan_types">Scan types</a> \
            &nbsp;&nbsp;|&nbsp;&nbsp;<a href="#hosts_targets">Hosts/Targets</a> \
            &nbsp;&nbsp;|&nbsp;&nbsp;<a href="#tcp_ports_analysis">TCP ports analysis</a> \
            &nbsp;&nbsp;|&nbsp;&nbsp;<a href="#udp_ports_analysis">UDP ports analysis</a> \
            </div>\n')

        #===== General information
        starttime, endtime, duration = self.ffts, self.lfts, (datetime.datetime.strptime(self.lfts, '%Y-%m-%d %H:%M:%S') - datetime.datetime.strptime(self.ffts, '%Y-%m-%d %H:%M:%S'))
        self.writefile("report.html", '<a name="general_information" id="general_information"></a>')
        self.writefile("report.html", '<div style="float:left">\n')
        self.writefile("report.html", "<h1>General information</h1>\n")
        self.writefile("report.html", '<table border="1">\n')
        self.writefile("report.html", '<tr><th>File name (MD5)</th><td>%s (%s)</td></tr>\n' % (self.pcapfile, self.md5sum(self.pcapfile)) )
        self.writefile("report.html", '<tr><th>Number of packets</th><td>%s</td></th></tr>\n' % self.countnumberframes())
        self.writefile("report.html", '<tr><th>Capture duration</th><td>%s</td></th></tr>\n' % duration)
        self.writefile("report.html", '<tr><th>Start time</th><td>%s</td></th></tr>\n' % starttime)
        self.writefile("report.html", '<tr><th>End time</th><td>%s</td></th></tr>\n' % endtime)
        self.writefile("report.html", '</table>\n')
        self.writefile("report.html", '</div>\n')

        #===== Discovered hosts
        self.writefile("report.html", '<a name="discovered_hosts" id="discovered_hosts"></a>')
        self.writefile("report.html", '<div style="float:left">\n')
        self.writefile("report.html", "<h1>Discovered hosts</h1>\n")
        self.writefile("report.html", '<table border="1">\n')
        self.writefile("report.html", '<tr><th>Host</th><th>MAC addr.</th><th>Vendor</th></tr>\n')
        for i in self.hosts:
            self.writefile("report.html", '<tr><td><a href="%s/graph.php?f=rcv_ip_id&dst=%s">%s</a> \
                </td><td>%s</td><td>%s</td></tr>\n' \
                % (self.BASEWB, i, i, self.hosts[i], self.macvendor(self.hosts[i])))
        self.writefile("report.html", "</table>\n")
        self.writefile("report.html", '</div>\n')

        #===== Scan types
        self.writefile("report.html", '<a name="scan_types" id="scan_types"></a>')
        self.writefile("report.html", '<div style="clear:both"></div><div style="float:left">\n')
        self.writefile("report.html", "<h1>Scan types</h1>\n")
        self.cursor.execute("""
            SELECT scan_type, COUNT( 1 )
            FROM pyscan
            GROUP BY scan_type
            ORDER BY COUNT( 1 ) DESC
            """)
        self.writefile("report.html", '<table border="1">\n')
        self.writefile("report.html", '<tr><th>Scan type</th><th># scans</th></tr>\n')
        while 1:
            row = self.cursor.fetchone()
            if row == None:
                break
            self.writefile("report.html", '<tr><td><a href="%s/pyscan.php?scan_type=%s">%s</a> \
                </td><td align="right">%s</td></tr>\n' \
                % (self.BASEWB, row[0].replace(" ", "_"), row[0], row[1]))

        self.writefile("report.html", "</table>\n")
        self.writefile("report.html", '<div><a href="%s/distributions.php">Scan distrib. charact.</a></div>\n' % (self.BASEWB))
        self.writefile("report.html", '</div>\n')

        #===== IP Options
        self.writefile("report.html", '<a name="scan_types" id="scan_types"></a>')
        self.writefile("report.html", '<div style="float:left">\n')
        self.writefile("report.html", "<h1>IP Options</h1>\n")
        self.cursor.execute("""
            SELECT ip_flags, count(1) FROM pyscan GROUP BY ip_flags
            """)
        self.writefile("report.html", '<table border="1">\n')
        self.writefile("report.html", '<tr><th>IP flag</th><th>Count</th></tr>\n')
        while 1:
            row = self.cursor.fetchone()
            if row == None:
                break
            self.writefile("report.html", '<tr><td>%s</td><td align="right">%s</td></tr>\n' \
                % (row[0], row[1]))

        self.writefile("report.html", "</table>\n")
        self.writefile("report.html", '</div>\n')

        #===== Hosts/Targets
        self.writefile("report.html", '<a name="hosts_targets" id="hosts_targets"></a>')
        self.writefile("report.html", '<div style="float:left">\n')
        self.writefile("report.html", "<h1>Hosts & targets</h1>\n")
        self.cursor.execute("""
            SELECT ip_src, ip_dst, COUNT(snd_frame)
            FROM pyscan
            GROUP BY ip_src, ip_dst
            ORDER BY ip_src, ip_dst
            """)
        self.writefile("report.html", '<table border="1">\n')
        self.writefile("report.html", '<tr><th>Src IP -> Dst IP</th><th># scans</th></tr>\n')
        while 1:
            row = self.cursor.fetchone()
            if row == None:
                break
            self.writefile("report.html", '<tr><td>%s -> %s</td><td align="right">%s</td></tr>\n' % (row[0], row[1], row[2]))
        self.writefile("report.html", "</table>\n")
        self.writefile("report.html", '</div>\n')
        self.writefile("report.html", '<div style="clear:both"></div>\n')

        #===== TCP ports analysis for each host
        self.writefile("report.html", '<a name="tcp_ports_analysis" id="tcp_ports_analysis"></a>')
        self.writefile("report.html", '<h1>TCP ports analysis</h1>\n')
        self.writefile("report.html", '<div style="clear:both"></div>\n')
        self.cursor.execute("""
            SELECT DISTINCT ip_dst, dport, state
            FROM pyscan
            WHERE proto='TCP'
            ORDER BY ip_dst, dport, state
        """)
        ip, port = '', ''
        state = []
        while 1:
            row = self.cursor.fetchone()
            if row == None:
                break

            # For each new IP: a new table
            if row[0]!=ip:
                if ip!='':
                    # End of previous table
                    self.writefile("report.html", '<tr><td>%s/tcp</td><td>%s</td></tr>\n' \
                        % (port, "/".join(state)) )
                    port = ''
                    self.writefile("report.html", "</table>\n</div>\n")
                self.writefile("report.html", '<div style="float:left">\n \
                    <h2>%s</h2>\n \
                    <table>\n \
                    <tr><th>Port</th><th>State</th></tr>\n' \
                    % row[0])
                ip = row[0]

            # For each new port
            if row[1]!=port:
                # We dump the list of states for port
                if port!='':
                    self.writefile("report.html", '<tr><td>%s/tcp</td><td>%s</td></tr>\n' \
                        % (port, "/".join(state)) )
                state = [] # We reset state list
                port = row[1]

            # Gathers all different states for given port
            state.append('<a href="%s/pyscan.php?ip=%s&proto=TCP&port=%s&state=%s">%s</a>' \
                % (self.BASEWB, ip, port, row[2], row[2]))

        # We dump content for last table of TCP section
        self.writefile("report.html", '<tr><td>%s/tcp</td><td>%s</td></tr>\n' \
            % (port, "/".join(state)) )
        self.writefile("report.html", '</table>\n</div>\n<div style="clear:both"></div>')

        #===== UDP ports analysis for each host
        self.writefile("report.html", '<a name="udp_ports_analysis" id="udp_ports_analysis"></a>')
        self.writefile("report.html", '<h1>UDP ports analysis</h1>\n')
        self.writefile("report.html", '<div style="clear:both"></div>\n')
        self.cursor.execute("""
            SELECT DISTINCT ip_dst, dport, state
            FROM pyscan
            WHERE proto='UDP'
            ORDER BY ip_dst, dport, state
        """)
        ip, port = '', ''
        state = []
        while 1:
            row = self.cursor.fetchone()
            if row == None:
                break

            # For each new IP: a new table
            if row[0]!=ip:
                if ip!='':
                    # End of previous table: we dump list of states for port
                    self.writefile("report.html", '<tr><td>%s/udp</td><td>%s</td></tr>\n' \
                        % (port, "/".join(state)) )
                    port = ''
                    self.writefile("report.html", "</table>\n</div>\n")
                self.writefile("report.html", '<div style="float:left">\n \
                    <h2>%s</h2>\n \
                    <table>\n \
                    <tr><th>Port</th><th>State</th></tr>\n' \
                    % row[0])
                ip = row[0]

            # For each new port
            if row[1]!=port:
                # We dump the list of states for port
                if port!='':
                    self.writefile("report.html", '<tr><td>%s/udp</td><td>%s</td></tr>\n' \
                        % (port, "/".join(state)) )
                state = [] # We reset state list
                port = row[1]

            # Gathers all different states for given port
            state.append('<a href="%s/pyscan.php?ip=%s&proto=UDP&port=%s&state=%s">%s</a>' \
                % (self.BASEWB, ip, port, row[2], row[2]))

        # We dump content for last table of UDP section
        self.writefile("report.html", '<tr><td>%s/udp</td><td>%s</td></tr>\n' \
            % (port, "/".join(state)) )
        self.writefile("report.html", '</table>\n</div>\n<div style="clear:both"></div>')

        # End of report
        self.writefile("report.html", "</body>\n</html>\n")
        
        
    def closeconnection(self):
        """Closes MySQL connection
        """
        # Closes MySQL connection and cursor
        self.cursor.close ()
        self.conn.close ()


if __name__ == '__main__':
    usage = "usage: %prog -r <file> [options]"
    parser = OptionParser(usage)
    parser.add_option("-r", "--read-file", dest="pcap_file",
        help="Capture file to process (pcap format)")
    parser.add_option("-o", '--output', dest="output_directory", default="./report",
        help="Reporting directory (default: ./report/)")
    parser.add_option("-f", '--force', dest="force", default=False, action="store_true",
        help="Force overwriting of files")
    parser.add_option("-v", '--vendor-database', dest="oui_file", default="./oui.txt",
        help="Vendor database (default: ./oui.txt)")
    parser.add_option("-d", "--dont-purge", dest="dont_purge", default=False, action="store_true",
        help="Don't purge existing data")
    (options, args) = parser.parse_args(sys.argv)
    # pcap is mandatory
    if not options.pcap_file:
        parser.error("Capture file is missing. Use -r <file>.")
    # Checks if output directory does not already exist as file
    if options.output_directory and os.path.isfile(options.output_directory):
        parser.error("Use a different name for output directory since it is already used for a file")
    # Checks that --force option is used if output directory exists
    if options.output_directory and os.path.isdir(options.output_directory) and not options.force:
        if os.listdir(options.output_directory):
            parser.error("Output directory is not empty. Use -f to overwrite content")
    # If force option is called, removes output directory
    if options.force:
        shutil.rmtree(options.output_directory, ignore_errors=True)
    
    p = PyScan(options.pcap_file, options.output_directory, options.oui_file, options.dont_purge)
    p.main()
    p.updatescantypes()
    p.exportresults()
    p.closeconnection()
    print "Finished. See generated report for full analysis.\n"
    del p
